@@ -269,6 +269,20 @@ int AH_Job_VPP_IsNeededForCode(const AH_JOB *j, const char *code)
 
 
 
+int AH_Job_VPP_IsResponseStructured(const AH_JOB *j)
+{
+  GWEN_DB_NODE *dbParams;
+  const char *s;
+
+  dbParams=AH_Job_GetParams(j);
+  assert(dbParams);
+  s=GWEN_DB_GetCharValue(dbParams, "textIsStructured", 0, "N");
+  return (s && *s && toupper(*s)=='J')?1:0;
+}
+
+
+
+
 void _readVopId(AH_JOB_VPP *aj, GWEN_DB_NODE *dbArgs, GWEN_DB_NODE *dbVppResponse)
 {
   const uint8_t *ptr;
@@ -337,6 +351,7 @@ void _readVopResultGroup(AH_JOB *j, AH_JOB_VPP *aj, GWEN_DB_NODE *dbVppResponse)
       AH_VopResult_SetResult(vr, vopResultCode);
       AH_VopResult_List_Add(vr, aj->resultList);
       DBG_ERROR(AQHBCI_LOGDOMAIN, "Added VOP result");
+      AH_VopResult_Log(vr, AQHBCI_LOGDOMAIN, GWEN_LoggerLevel_Error); /* use ERROR for now */
     }
 
     dbVppResult=GWEN_DB_FindNextGroup(dbVppResult, "vopResult");
@@ -393,6 +408,7 @@ void _addResultsFromTransactionList(AH_JOB *j, AH_JOB_VPP *aj, AB_TRANSACTION_LI
 	AH_VopResult_SetResult(vr, vopResultCode);
 	AH_VopResult_List_Add(vr, aj->resultList);
 	DBG_ERROR(AQHBCI_LOGDOMAIN, "Added VOP result");
+        AH_VopResult_Log(vr, AQHBCI_LOGDOMAIN, GWEN_LoggerLevel_Error); /* use ERROR for now */
       }
       else {
 	DBG_ERROR(AQHBCI_LOGDOMAIN, "Not adding incomplete VOP result (pending)");
